@@ -20,31 +20,38 @@ import tomlkit
 import vistir
 
 from pipenv.cmdparse import Script
-from pipenv.core import system_which
 from pipenv.environment import Environment
 from pipenv.environments import Setting, is_in_virtualenv, normalize_pipfile_path
+from pipenv.patched.pip._internal.commands.install import InstallCommand
 from pipenv.utils.constants import is_type_checking
 from pipenv.utils.dependencies import (
     get_canonical_names,
     is_editable,
     is_star,
+    pep423_name,
     python_version,
 )
 from pipenv.utils.internet import get_url_name, is_valid_url, proper_case
-from pipenv.utils.resolver import pep423_name
 from pipenv.utils.shell import (
     find_requirements,
     find_windows_executable,
-    get_pipenv_dist,
     get_workon_home,
     is_virtual_environment,
     looks_like_dir,
     safe_expandvars,
+    system_which,
 )
 from pipenv.utils.toml import cleanup_toml, convert_toml_outline_tables
-from pipenv.vendor.cached_property import cached_property
 from pipenv.vendor.requirementslib.models.utils import get_default_pyproject_backend
 
+try:
+    # this is only in Python3.8 and later
+    from functools import cached_property
+except ImportError:
+    # eventually distlib will remove cached property when they drop Python3.7
+    from pipenv.patched.pip._vendor.distlib.util import cached_property
+
+
 if is_type_checking():
     from typing import Dict, List, Optional, Set, Text, Tuple, Union
 
@@ -288,11 +295,6 @@ class Project:
             pipfile=self.parsed_pipfile,
             project=self,
         )
-        pipenv_dist = get_pipenv_dist(pkg="pipenv")
-        if pipenv_dist:
-            environment.extend_dists(pipenv_dist)
-        else:
-            environment.add_dist("pipenv")
         return environment
 
     @property
@@ -364,6 +366,9 @@ class Project:
 
     @property
     def virtualenv_name(self) -> str:
+        custom_name = self.s.PIPENV_CUSTOM_VENV_NAME
+        if custom_name:
+            return custom_name
         sanitized, encoded_hash = self._get_virtualenv_hash(self.name)
         suffix = ""
         if self.s.PIPENV_PYTHON:
@@ -574,7 +579,7 @@ class Project:
     def lockfile_content(self):
         return self.load_lockfile()
 
-    def _get_editable_packages(self, dev=False):
+    def get_editable_packages(self, dev=False):
         section = "dev-packages" if dev else "packages"
         packages = {
             k: v
@@ -596,11 +601,11 @@ class Project:
 
     @property
     def editable_packages(self):
-        return self._get_editable_packages(dev=False)
+        return self.get_editable_packages(dev=False)
 
     @property
     def editable_dev_packages(self):
-        return self._get_editable_packages(dev=True)
+        return self.get_editable_packages(dev=True)
 
     @property
     def vcs_packages(self):
@@ -641,10 +646,8 @@ class Project:
 
     def create_pipfile(self, python=None):
         """Creates the Pipfile, filled with juicy defaults."""
-        from .vendor.pip_shims.shims import InstallCommand
-
         # Inherit the pip's index configuration of install command.
-        command = InstallCommand()
+        command = InstallCommand(name="InstallCommand", summary="pip Install command.")
         indexes = command.cmd_opts.get_option("--extra-index-url").default
         sources = [self.default_source]
         for i, index in enumerate(indexes):
