@@ -1,8 +1,10 @@
 import os
 from contextlib import contextmanager
+from tempfile import NamedTemporaryFile
 from typing import Mapping, Sequence
 
-from pipenv.patched.notpip._vendor.packaging.markers import Marker
+from pipenv.patched.pip._vendor.packaging.markers import Marker
+from pipenv.patched.pip._vendor.packaging.version import parse
 
 from .constants import SCHEME_LIST, VCS_LIST
 from .shell import temp_path
@@ -49,7 +51,7 @@ class HackedPythonVersion:
 
 def get_canonical_names(packages):
     """Canonicalize a list of packages and return a set of canonical names"""
-    from pipenv.patched.notpip._vendor.packaging.utils import canonicalize_name
+    from pipenv.patched.pip._vendor.packaging.utils import canonicalize_name
 
     if not isinstance(packages, Sequence):
         if not isinstance(packages, str):
@@ -60,10 +62,7 @@ def get_canonical_names(packages):
 
 def pep440_version(version):
     """Normalize version to PEP 440 standards"""
-    # Use pip built-in version parser.
-    from pipenv.vendor.pip_shims import shims
-
-    return str(shims.parse_version(version))
+    return str(parse(version))
 
 
 def pep423_name(name):
@@ -130,7 +129,7 @@ def translate_markers(pipfile_entry):
     """
     if not isinstance(pipfile_entry, Mapping):
         raise TypeError("Entry is not a pipfile formatted mapping.")
-    from pipenv.patched.notpip._vendor.packaging.markers import default_environment
+    from pipenv.patched.pip._vendor.packaging.markers import default_environment
 
     allowed_marker_keys = ["markers"] + list(default_environment().keys())
     provided_keys = list(pipfile_entry.keys()) if hasattr(pipfile_entry, "keys") else []
@@ -273,14 +272,65 @@ def convert_deps_to_pip(
         return dependencies
 
     # Write requirements.txt to tmp directory.
-    from pipenv.vendor.vistir.path import create_tracked_tempfile
-
-    f = create_tracked_tempfile(suffix="-requirements.txt", delete=False)
+    f = NamedTemporaryFile(suffix="-requirements.txt", delete=False)
     f.write("\n".join(dependencies).encode("utf-8"))
     f.close()
     return f.name
 
 
+def get_constraints_from_deps(deps):
+    """Get contraints from Pipfile-formatted dependency"""
+    from pipenv.patched.pip._internal.req.req_install import (
+        check_invalid_constraint_type,
+    )
+    from pipenv.vendor.requirementslib.models.requirements import Requirement
+
+    constraints = []
+    for dep_name, dep in deps.items():
+        new_dep = Requirement.from_pipfile(dep_name, dep)
+        problem = check_invalid_constraint_type(new_dep.as_ireq())
+        if not problem:
+            c = new_dep.as_line().strip()
+            constraints.append(c)
+    return constraints
+
+
+def prepare_constraint_file(
+    constraints,
+    directory=None,
+    sources=None,
+    pip_args=None,
+):
+    from pipenv.vendor.vistir.path import (
+        create_tracked_tempdir,
+        create_tracked_tempfile,
+    )
+
+    if not directory:
+        directory = create_tracked_tempdir(suffix="-requirements", prefix="pipenv-")
+
+    constraints_file = create_tracked_tempfile(
+        mode="w",
+        prefix="pipenv-",
+        suffix="-constraints.txt",
+        dir=directory,
+        delete=False,
+    )
+
+    if sources and pip_args:
+        skip_args = ("build-isolation", "use-pep517", "cache-dir")
+        args_to_add = [
+            arg for arg in pip_args if not any(bad_arg in arg for bad_arg in skip_args)
+        ]
+        requirementstxt_sources = " ".join(args_to_add) if args_to_add else ""
+        requirementstxt_sources = requirementstxt_sources.replace(" --", "\n--")
+        constraints_file.write(f"{requirementstxt_sources}\n")
+
+    constraints_file.write("\n".join([c for c in constraints]))
+    constraints_file.close()
+    return constraints_file.name
+
+
 def is_required_version(version, specified_version):
     """Check to see if there's a hard requirement for version
     number provided in the Pipfile.
@@ -308,12 +358,6 @@ def locked_repository(requirement):
 
     if not requirement.is_vcs:
         return
-    original_base = os.environ.pop("PIP_SHIMS_BASE_MODULE", None)
-    os.environ["PIP_SHIMS_BASE_MODULE"] = "pipenv.patched.notpip"
     src_dir = create_tracked_tempdir(prefix="pipenv-", suffix="-src")
-    try:
-        with requirement.req.locked_vcs_repo(src_dir=src_dir) as repo:
-            yield repo
-    finally:
-        if original_base:
-            os.environ["PIP_SHIMS_BASE_MODULE"] = original_base
+    with requirement.req.locked_vcs_repo(src_dir=src_dir) as repo:
+        yield repo
