@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 import contextlib
 import hashlib
 import os
@@ -7,23 +5,48 @@ import subprocess
 import sys
 import warnings
 from functools import lru_cache
+from typing import Dict, List, Optional, Set, Tuple, Union
 
 from pipenv import environments
 from pipenv.exceptions import RequirementError, ResolutionFailure
-from pipenv.utils.constants import MYPY_RUNNING
+from pipenv.patched.pip._internal.cache import WheelCache
+from pipenv.patched.pip._internal.commands.install import InstallCommand
+from pipenv.patched.pip._internal.exceptions import InstallationError
+from pipenv.patched.pip._internal.models.target_python import TargetPython
+from pipenv.patched.pip._internal.network.cache import SafeFileCache
+from pipenv.patched.pip._internal.operations.build.build_tracker import (
+    get_build_tracker,
+)
+from pipenv.patched.pip._internal.req.constructors import (
+    install_req_from_parsed_requirement,
+)
+from pipenv.patched.pip._internal.req.req_file import parse_requirements
+from pipenv.patched.pip._internal.utils.hashes import FAVORITE_HASH
+from pipenv.patched.pip._internal.utils.temp_dir import global_tempdir_manager
+from pipenv.project import Project
 from pipenv.vendor import click
-from pipenv.vendor.requirementslib import Pipfile, Requirement
+from pipenv.vendor.requirementslib import Requirement
+from pipenv.vendor.requirementslib.models.requirements import Line
 from pipenv.vendor.requirementslib.models.utils import DIRECT_URL_RE
 from pipenv.vendor.vistir import TemporaryDirectory, open_file
 from pipenv.vendor.vistir.path import create_tracked_tempdir
 
+try:
+    # this is only in Python3.8 and later
+    from functools import cached_property
+except ImportError:
+    # eventually distlib will remove cached property when they drop Python3.7
+    from pipenv.patched.pip._vendor.distlib.util import cached_property
+
 from .dependencies import (
     HackedPythonVersion,
     clean_pkg_version,
     convert_deps_to_pip,
+    get_constraints_from_deps,
     get_vcs_deps,
     is_pinned_requirement,
     pep423_name,
+    prepare_constraint_file,
     translate_markers,
 )
 from .indexes import parse_indexes, prepare_pip_source_args
@@ -32,11 +55,34 @@ from .locking import format_requirement_for_lockfile, prepare_lockfile
 from .shell import make_posix, subprocess_run, temp_environ
 from .spinner import create_spinner
 
-if MYPY_RUNNING:
-    from typing import Any, Dict, List, Optional, Set, Tuple, Union  # noqa
 
-    from pipenv.project import Project  # noqa
-    from pipenv.vendor.requirementslib.models.requirements import Line  # noqa
+def get_package_finder(
+    install_cmd=None,
+    options=None,
+    session=None,
+    platform=None,
+    python_versions=None,
+    abi=None,
+    implementation=None,
+    ignore_requires_python=None,
+):
+    """Reduced Shim for compatibility to generate package finders."""
+    py_version_info = None
+    if python_versions:
+        py_version_info_python = max(python_versions)
+        py_version_info = tuple([int(part) for part in py_version_info_python])
+    target_python = TargetPython(
+        platforms=[platform] if platform else None,
+        py_version_info=py_version_info,
+        abis=[abi] if abi else None,
+        implementation=implementation,
+    )
+    return install_cmd._build_package_finder(
+        options=options,
+        session=session,
+        target_python=target_python,
+        ignore_requires_python=ignore_requires_python,
+    )
 
 
 class HashCacheMixin:
@@ -64,9 +110,7 @@ class HashCacheMixin:
         return hash_value.decode("utf8")
 
     def _get_file_hash(self, link):
-        from pipenv.vendor.pip_shims import shims
-
-        h = hashlib.new(shims.FAVORITE_HASH)
+        h = hashlib.new(FAVORITE_HASH)
         with open_file(link.url, self.session) as fp:
             for chunk in iter(lambda: fp.read(8096), b""):
                 h.update(chunk)
@@ -85,6 +129,7 @@ class Resolver:
         skipped=None,
         clear=False,
         pre=False,
+        dev=False,
     ):
         self.initial_constraints = constraints
         self.req_dir = req_dir
@@ -94,6 +139,7 @@ class Resolver:
         self.hashes = {}
         self.clear = clear
         self.pre = pre
+        self.dev = dev
         self.results = None
         self.markers_lookup = markers_lookup if markers_lookup is not None else {}
         self.index_lookup = index_lookup if index_lookup is not None else {}
@@ -122,23 +168,18 @@ class Resolver:
     @staticmethod
     @lru_cache()
     def _get_pip_command():
-        from pipenv.vendor.pip_shims import shims
-
-        return shims.InstallCommand()
+        return InstallCommand(name="InstallCommand", summary="pip Install command.")
 
     @property
     def hash_cache(self):
-        from pipenv.vendor.pip_shims import shims
-
         if not self._hash_cache:
-            self._hash_cache = type(
-                "HashCache", (HashCacheMixin, shims.SafeFileCache), {}
-            )(os.path.join(self.project.s.PIPENV_CACHE_DIR, "hashes"), self.session)
+            self._hash_cache = type("HashCache", (HashCacheMixin, SafeFileCache), {})(
+                os.path.join(self.project.s.PIPENV_CACHE_DIR, "hashes"), self.session
+            )
         return self._hash_cache
 
-    @classmethod
     def get_metadata(
-        cls,
+        self,
         deps: List[str],
         index_lookup: Dict[str, str],
         markers_lookup: Dict[str, str],
@@ -161,7 +202,7 @@ class Resolver:
             markers_lookup = {}
         if not req_dir:
             req_dir = create_tracked_tempdir(prefix="pipenv-", suffix="-reqdir")
-        transient_resolver = cls(
+        transient_resolver = Resolver(
             [],
             req_dir,
             project,
@@ -174,7 +215,7 @@ class Resolver:
         for dep in deps:
             if not dep:
                 continue
-            req, req_idx, markers_idx = cls.parse_line(
+            req, req_idx, markers_idx = self.parse_line(
                 dep,
                 index_lookup=index_lookup,
                 markers_lookup=markers_lookup,
@@ -193,7 +234,7 @@ class Resolver:
                 )
             if not use_sources:
                 use_sources = sources
-            transient_resolver = cls(
+            transient_resolver = Resolver(
                 [],
                 req_dir,
                 project,
@@ -203,16 +244,15 @@ class Resolver:
                 clear=clear,
                 pre=pre,
             )
-            constraint_update, lockfile_update = cls.get_deps_from_req(
+            constraint_update, lockfile_update = self.get_deps_from_req(
                 req, resolver=transient_resolver, resolve_vcs=project.s.PIPENV_RESOLVE_VCS
             )
             constraints |= constraint_update
             skipped.update(lockfile_update)
         return constraints, skipped, index_lookup, markers_lookup
 
-    @classmethod
     def parse_line(
-        cls,
+        self,
         line: str,
         index_lookup: Dict[str, str] = None,
         markers_lookup: Dict[str, str] = None,
@@ -264,9 +304,8 @@ class Resolver:
             markers_lookup[req.normalized_name] = req.markers.replace('"', "'")
         return req, index_lookup, markers_lookup
 
-    @classmethod
     def get_deps_from_req(
-        cls,
+        self,
         req: Requirement,
         resolver: Optional["Resolver"] = None,
         resolve_vcs: bool = True,
@@ -280,6 +319,7 @@ class Resolver:
         # TODO: this is way too complex, refactor this
         constraints: Set[str] = set()
         locked_deps: Dict[str, Dict[str, Union[str, bool, List[str]]]] = {}
+        editable_packages = self.project.get_editable_packages(dev=self.dev)
         if (req.is_file_or_url or req.is_vcs) and not req.is_wheel:
             # for local packages with setup.py files and potential direct url deps:
             if req.is_vcs:
@@ -315,21 +355,22 @@ class Resolver:
                         if not r.url:
                             continue
                         line = _requirement_to_str_lowercase_name(r)
-                        new_req, _, _ = cls.parse_line(line)
+                        new_req, _, _ = self.parse_line(line)
                         if r.marker and not r.marker.evaluate():
                             new_constraints = {}
                             _, new_entry = req.pipfile_entry
                             new_lock = {pep423_name(new_req.normalized_name): new_entry}
                         else:
-                            new_constraints, new_lock = cls.get_deps_from_req(
+                            new_constraints, new_lock = self.get_deps_from_req(
                                 new_req, resolver
                             )
                         locked_deps.update(new_lock)
                         constraints |= new_constraints
                 # if there is no marker or there is a valid marker, add the constraint line
                 elif r and (not r.marker or (r.marker and r.marker.evaluate())):
-                    line = _requirement_to_str_lowercase_name(r)
-                    constraints.add(line)
+                    if r.name not in editable_packages:
+                        line = _requirement_to_str_lowercase_name(r)
+                        constraints.add(line)
             # ensure the top level entry remains as provided
             # note that we shouldn't pin versions for editable vcs deps
             if not req.is_vcs:
@@ -392,6 +433,7 @@ class Resolver:
         req_dir: str = None,
         clear: bool = False,
         pre: bool = False,
+        dev: bool = False,
     ) -> "Resolver":
 
         if not req_dir:
@@ -402,7 +444,18 @@ class Resolver:
             markers_lookup = {}
         if sources is None:
             sources = project.sources
-        constraints, skipped, index_lookup, markers_lookup = cls.get_metadata(
+        resolver = Resolver(
+            [],
+            req_dir,
+            project,
+            sources,
+            index_lookup=index_lookup,
+            markers_lookup=markers_lookup,
+            clear=clear,
+            pre=pre,
+            dev=dev,
+        )
+        constraints, skipped, index_lookup, markers_lookup = resolver.get_metadata(
             deps,
             index_lookup,
             markers_lookup,
@@ -411,58 +464,12 @@ class Resolver:
             req_dir=req_dir,
             pre=pre,
             clear=clear,
-        )
-        return Resolver(
-            constraints,
-            req_dir,
-            project,
-            sources,
-            index_lookup=index_lookup,
-            markers_lookup=markers_lookup,
-            skipped=skipped,
-            clear=clear,
-            pre=pre,
-        )
-
-    @classmethod
-    def from_pipfile(
-        cls,
-        project: Optional[Project],
-        pipfile: Optional[Pipfile] = None,
-        dev: bool = False,
-        pre: bool = False,
-        clear: bool = False,
-    ) -> "Resolver":
-
-        if not pipfile:
-            pipfile = project._pipfile
-        req_dir = create_tracked_tempdir(suffix="-requirements", prefix="pipenv-")
-        index_lookup, markers_lookup = {}, {}
-        deps = set()
-        if dev:
-            deps.update({req.as_line() for req in pipfile.dev_packages})
-        deps.update({req.as_line() for req in pipfile.packages})
-        constraints, skipped, index_lookup, markers_lookup = cls.get_metadata(
-            list(deps),
-            index_lookup,
-            markers_lookup,
-            project,
-            project.sources,
-            req_dir=req_dir,
-            pre=pre,
-            clear=clear,
-        )
-        return Resolver(
-            constraints,
-            req_dir,
-            project,
-            project.sources,
-            index_lookup=index_lookup,
-            markers_lookup=markers_lookup,
-            skipped=skipped,
-            clear=clear,
-            pre=pre,
-        )
+        )  # Workaround to the fact `get_metadata` instantiates a transient Resolver
+        resolver.initial_constraints = constraints
+        resolver.skipped = skipped
+        resolver.index_lookup = index_lookup
+        resolver.markers_lookup = markers_lookup
+        return resolver
 
     @property
     def pip_command(self):
@@ -494,29 +501,13 @@ class Resolver:
         return self._pip_args
 
     def prepare_constraint_file(self):
-        from pipenv.vendor.vistir.path import create_tracked_tempfile
-
-        constraints_file = create_tracked_tempfile(
-            mode="w",
-            prefix="pipenv-",
-            suffix="-constraints.txt",
-            dir=self.req_dir,
-            delete=False,
+        constraint_filename = prepare_constraint_file(
+            self.initial_constraints,
+            directory=self.req_dir,
+            sources=self.sources,
+            pip_args=self.pip_args,
         )
-        skip_args = ("build-isolation", "use-pep517", "cache-dir")
-        args_to_add = [
-            arg
-            for arg in self.pip_args
-            if not any(bad_arg in arg for bad_arg in skip_args)
-        ]
-        if self.sources:
-            requirementstxt_sources = " ".join(args_to_add) if args_to_add else ""
-            requirementstxt_sources = requirementstxt_sources.replace(" --", "\n--")
-            constraints_file.write(f"{requirementstxt_sources}\n")
-        constraints = self.initial_constraints
-        constraints_file.write("\n".join([c for c in constraints]))
-        constraints_file.close()
-        return constraints_file.name
+        return constraint_filename
 
     @property
     def constraint_file(self):
@@ -524,6 +515,17 @@ class Resolver:
             self._constraint_file = self.prepare_constraint_file()
         return self._constraint_file
 
+    @cached_property
+    def default_constraint_file(self):
+        default_constraints = get_constraints_from_deps(self.project.packages)
+        default_constraint_filename = prepare_constraint_file(
+            default_constraints,
+            directory=self.req_dir,
+            sources=None,
+            pip_args=None,
+        )
+        return default_constraint_filename
+
     @property
     def pip_options(self):
         if self._pip_options is None:
@@ -558,10 +560,8 @@ class Resolver:
 
     @property
     def finder(self):
-        from pipenv.vendor.pip_shims import shims
-
         if self._finder is None:
-            self._finder = shims.get_package_finder(
+            self._finder = get_package_finder(
                 install_cmd=self.pip_command,
                 options=self.pip_options,
                 session=self.session,
@@ -573,10 +573,8 @@ class Resolver:
 
     @property
     def ignore_compatibility_finder(self):
-        from pipenv.vendor.pip_shims import shims
-
         if self._ignore_compatibility_finder is None:
-            ignore_compatibility_finder = shims.get_package_finder(
+            ignore_compatibility_finder = get_package_finder(
                 install_cmd=self.pip_command,
                 options=self.pip_options,
                 session=self.session,
@@ -595,12 +593,10 @@ class Resolver:
 
     @property
     def parsed_constraints(self):
-        from pipenv.vendor.pip_shims import shims
-
         pip_options = self.pip_options
         pip_options.extra_index_urls = []
         if self._parsed_constraints is None:
-            self._parsed_constraints = shims.parse_requirements(
+            self._parsed_constraints = parse_requirements(
                 self.constraint_file,
                 finder=self.finder,
                 session=self.session,
@@ -608,12 +604,33 @@ class Resolver:
             )
         return self._parsed_constraints
 
+    @cached_property
+    def parsed_default_constraints(self):
+        pip_options = self.pip_options
+        pip_options.extra_index_urls = []
+        parsed_default_constraints = parse_requirements(
+            self.default_constraint_file,
+            constraint=True,
+            finder=self.finder,
+            session=self.session,
+            options=pip_options,
+        )
+        return parsed_default_constraints
+
+    @cached_property
+    def default_constraints(self):
+        default_constraints = [
+            install_req_from_parsed_requirement(
+                c,
+                isolated=self.pip_options.build_isolation,
+                user_supplied=False,
+            )
+            for c in self.parsed_default_constraints
+        ]
+        return default_constraints
+
     @property
     def constraints(self):
-        from pipenv.patched.notpip._internal.req.constructors import (
-            install_req_from_parsed_requirement,
-        )
-
         if self._constraints is None:
             self._constraints = [
                 install_req_from_parsed_requirement(
@@ -624,16 +641,13 @@ class Resolver:
                 )
                 for c in self.parsed_constraints
             ]
+            # Only use default_constraints when installing dev-packages
+            if self.dev:
+                self._constraints += self.default_constraints
         return self._constraints
 
     @contextlib.contextmanager
     def get_resolver(self, clear=False):
-        from pipenv.vendor.pip_shims.shims import (
-            WheelCache,
-            get_build_tracker,
-            global_tempdir_manager,
-        )
-
         with global_tempdir_manager(), get_build_tracker() as build_tracker, TemporaryDirectory(
             suffix="-build", prefix="pipenv-"
         ) as directory:
@@ -664,9 +678,6 @@ class Resolver:
             yield resolver
 
     def resolve(self):
-        from pipenv.exceptions import ResolutionFailure
-        from pipenv.vendor.pip_shims.shims import InstallationError
-
         self.constraints  # For some reason it is important to evaluate constraints before resolver context
         with temp_environ(), self.get_resolver() as resolver:
             try:
@@ -712,8 +723,6 @@ class Resolver:
         return cleaned_checksums
 
     def _get_hashes_from_pypi(self, ireq):
-        from pipenv.vendor.pip_shims import shims
-
         pkg_url = f"https://pypi.org/pypi/{ireq.name}/json"
         session = _get_requests_session(self.project.s.PIPENV_MAX_RETRIES)
         try:
@@ -731,13 +740,13 @@ class Resolver:
                 if spec:
                     version = spec.version
             for release in cleaned_releases[version]:
-                collected_hashes.add(release["digests"][shims.FAVORITE_HASH])
-            return self.prepend_hash_types(collected_hashes, shims.FAVORITE_HASH)
+                collected_hashes.add(release["digests"][FAVORITE_HASH])
+            return self.prepend_hash_types(collected_hashes, FAVORITE_HASH)
         except (ValueError, KeyError, ConnectionError):
             if self.project.s.is_verbose():
                 click.echo(
                     "{}: Error generating hash for {}".format(
-                        click.style("Warning", bold=True, fg="Red"), ireq.name
+                        click.style("Warning", bold=True, fg="red"), ireq.name
                     ),
                     err=True,
                 )
@@ -783,9 +792,7 @@ class Resolver:
         return self.hashes
 
     def _get_hash_from_link(self, link):
-        from pipenv.vendor.pip_shims import shims
-
-        if link.hash and link.hash_name == shims.FAVORITE_HASH:
+        if link.hash and link.hash_name == FAVORITE_HASH:
             return f"{link.hash_name}:{link.hash}"
 
         return self.hash_cache.get_hash(link)
@@ -861,6 +868,7 @@ def actually_resolve_deps(
     sources,
     clear,
     pre,
+    dev,
     req_dir=None,
 ):
     if not req_dir:
@@ -869,7 +877,7 @@ def actually_resolve_deps(
 
     with warnings.catch_warnings(record=True) as warning_list:
         resolver = Resolver.create(
-            deps, project, index_lookup, markers_lookup, sources, req_dir, clear, pre
+            deps, project, index_lookup, markers_lookup, sources, req_dir, clear, pre, dev
         )
         resolver.resolve()
         hashes = resolver.resolve_hashes()
@@ -1055,6 +1063,7 @@ def resolve_deps(
     python=False,
     clear=False,
     pre=False,
+    dev=False,
     allow_global=False,
     req_dir=None,
 ):
@@ -1085,6 +1094,7 @@ def resolve_deps(
                 sources,
                 clear,
                 pre,
+                dev,
                 req_dir=req_dir,
             )
         except RuntimeError:
@@ -1113,6 +1123,7 @@ def resolve_deps(
                     sources,
                     clear,
                     pre,
+                    dev,
                     req_dir=req_dir,
                 )
             except RuntimeError:
